
# q-expansion of newform 1600.3.g.h, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1600
weight = 3
poly_data = [985, 4, 164, -356, 229, -64, 26, -4, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[-560, 1576, -1772, 400, -220, 24, -8, 0], 975], [[-44, 171, -189, 37, -21, 3, -1, 0], 39], [[1460, -6, -18, 50, -30, 6, -2, 0], 75], [[4130, -6763, -4683, 3773, -1015, 483, -77, 22], 7575], [[-20840, 51706, -31002, 23782, -4790, 2154, -238, 68], 19695], [[-64590, 158864, -92256, 72336, -16780, 7776, -1064, 304], 32825], [[-2820, 4588, 3314, -2744, 830, -388, 56, -16], 1515]]

hecke_ring_character_values = [[1151, [-1, 0, 0, 0, 0, 0, 0, 0]], [901, [-1, 0, 0, 0, 0, 0, 0, 0]], [577, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -1], [0, 0, 0, 0, 0, 4, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 3], [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 4, 0], [0, -1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -3, 0, 0], [-24, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 23, 0, 0, 0], [0, 0, -1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 10, 0, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, 0, 0, 0, 0, 0, 3, 0], [0, 0, 0, 0, 3, 0, 0, 0], [0, -21, 0, 0, 0, 0, 0, 0], [0, 0, 0, 5, 0, 0, 0, 0], [0, -25, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -7, 0, 0, 0], [-150, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 8, 0], [0, 0, -3, 0, 0, 0, 0, 0], [0, 0, 0, 0, -61, 0, 0, 0], [0, 0, 0, 0, 0, 0, -21, 0], [0, 0, 0, -8, 0, 0, 0, 0], [0, 0, 5, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -11], [0, 0, 0, -6, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -27], [0, 0, 0, 0, 0, 0, 11, 0], [0, -40, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 7, 0, 0], [0, 0, 0, 0, -49, 0, 0, 0], [0, 0, 27, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -5, 0, 0], [0, 0, 0, 0, 0, 0, 0, 29], [0, 0, 0, 0, 0, 0, 0, 0], [0, -6, 0, 0, 0, 0, 0, 0], [0, 0, 0, 5, 0, 0, 0, 0], [0, 0, 0, 0, 0, -70, 0, 0], [0, -47, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -3], [0, 0, -21, 0, 0, 0, 0, 0], [0, 0, 0, 0, 33, 0, 0, 0], [0, 0, 0, 0, 0, 0, -12, 0], [0, 0, 0, -15, 0, 0, 0, 0], [0, -27, 0, 0, 0, 0, 0, 0], [-28, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -29], [0, 0, 0, 14, 0, 0, 0, 0], [0, 0, 27, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -7, 0], [0, 107, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -15, 0, 0], [-204, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 165, 0, 0, 0], [0, 0, 0, 0, 0, -33, 0, 0], [0, 0, 0, 0, -205, 0, 0, 0], [0, -102, 0, 0, 0, 0, 0, 0], [0, 0, 0, -14, 0, 0, 0, 0], [0, 0, 0, 0, 0, -38, 0, 0], [0, 0, 0, 0, 0, 0, 0, 33], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 169, 0, 0, 0], [0, 0, 0, 0, 0, 0, -24, 0], [0, 0, 0, 10, 0, 0, 0, 0], [0, 18, 0, 0, 0, 0, 0, 0], [0, 0, 49, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 19, 0, 0], [0, 0, 0, 0, 0, 0, 0, -39], [0, 0, 23, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -17, 0], [0, 0, 0, 0, 0, 94, 0, 0], [-330, 0, 0, 0, 0, 0, 0, 0], [536, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -55], [0, 0, 0, 0, 0, 0, 57, 0], [0, 45, 0, 0, 0, 0, 0, 0], [0, 0, 0, -17, 0, 0, 0, 0], [0, 34, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -75, 0, 0, 0], [684, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 4, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 40, 0], [0, 0, 53, 0, 0, 0, 0, 0], [0, 0, 0, 0, -43, 0, 0, 0], [0, 138, 0, 0, 0, 0, 0, 0], [0, 0, 21, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -55], [0, 0, 0, 0, 0, 0, 0, 33], [0, 0, -67, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 92, 0], [-102, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -235, 0, 0, 0], [0, 0, 0, 0, 0, 0, -96, 0], [0, 0, 0, 0, 229, 0, 0, 0], [0, 0, 0, 0, 0, -161, 0, 0], [0, 0, 0, 0, -61, 0, 0, 0], [144, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -57], [0, 0, 0, -18, 0, 0, 0, 0], [0, 0, 0, 0, -355, 0, 0, 0], [0, 0, 0, 10, 0, 0, 0, 0], [0, 75, 0, 0, 0, 0, 0, 0], [740, 0, 0, 0, 0, 0, 0, 0], [0, 0, -57, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 88, 0, 0], [0, 0, 0, -5, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -21], [0, -80, 0, 0, 0, 0, 0, 0], [-420, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -427, 0, 0, 0], [0, 0, -51, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -104, 0, 0], [0, 0, 0, 0, 0, 0, 0, -59], [0, 0, 0, 0, 0, 0, 3, 0], [0, 0, 0, -25, 0, 0, 0, 0], [0, 0, 0, 0, 0, 228, 0, 0], [0, 0, 0, 0, -513, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -123], [0, 0, 0, 0, 0, 0, 5, 0], [0, 0, 0, 0, 0, 0, 24, 0], [0, -153, 0, 0, 0, 0, 0, 0], [0, 0, 125, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 45, 0, 0], [0, 0, 0, 0, 0, 0, 0, 165], [0, 0, -13, 0, 0, 0, 0, 0], [0, 152, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -121, 0, 0], [-642, 0, 0, 0, 0, 0, 0, 0], [230, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 18, 0, 0], [0, 0, 0, 0, 95, 0, 0, 0], [0, 0, 0, 0, 0, 202, 0, 0], [-906, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 147], [0, 0, 0, 0, 0, 0, 47, 0], [0, 0, -35, 0, 0, 0, 0, 0], [0, 0, 0, 0, 69, 0, 0, 0], [0, 0, 0, 0, 0, 0, -15, 0], [0, -204, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -92, 0, 0], [0, 0, 0, -24, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -57], [0, 0, 5, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -47, 0, 0], [-1320, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -429, 0, 0, 0], [0, 0, -11, 0, 0, 0, 0, 0], [0, 0, 0, 0, 239, 0, 0, 0], [0, -141, 0, 0, 0, 0, 0, 0], [0, -281, 0, 0, 0, 0, 0, 0], [276, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 29, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 128, 0], [0, 0, 0, 0, -149, 0, 0, 0], [0, 0, 0, -16, 0, 0, 0, 0], [0, 0, -145, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -95], [0, 0, 0, 73, 0, 0, 0, 0], [0, 0, 79, 0, 0, 0, 0, 0], [0, 52, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 194, 0, 0], [-1178, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 288, 0, 0], [0, 0, 0, 0, 0, 0, 0, 73], [0, 0, 0, 0, 0, 0, -72, 0], [0, -426, 0, 0, 0, 0, 0, 0], [0, 0, 0, -45, 0, 0, 0, 0], [0, -352, 0, 0, 0, 0, 0, 0], [-282, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 39], [0, 0, 0, 0, 0, 0, -196, 0], [0, 0, -11, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 171, 0], [0, 0, 65, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 113], [0, 0, 0, 0, 0, -160, 0, 0], [0, 0, 0, -53, 0, 0, 0, 0], [0, 0, 105, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -184, 0], [0, 0, 0, 0, 0, 85, 0, 0], [0, 0, 0, 0, 313, 0, 0, 0], [-808, 0, 0, 0, 0, 0, 0, 0], [0, -498, 0, 0, 0, 0, 0, 0], [0, 0, 0, 12, 0, 0, 0, 0], [0, 0, 0, 0, 425, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -135], [0, 0, 0, 0, 0, 0, 161, 0], [0, 0, 0, 0, 543, 0, 0, 0], [0, 0, 0, -99, 0, 0, 0, 0], [-16, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -62, 0, 0], [0, 0, 0, -30, 0, 0, 0, 0], [0, 0, 93, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 268, 0], [0, 70, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -114, 0, 0], [1018, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 23], [0, 0, 0, 0, 0, 393, 0, 0], [0, 325, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -311, 0, 0, 0], [-2058, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 165], [0, 0, 0, -59, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 253, 0], [0, 0, 191, 0, 0, 0, 0, 0], [0, 0, 0, 0, 341, 0, 0, 0], [0, -225, 0, 0, 0, 0, 0, 0], [16, 0, 0, 0, 0, 0, 0, 0], [0, 0, -183, 0, 0, 0, 0, 0], [-474, 0, 0, 0, 0, 0, 0, 0], [0, 0, 143, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 12, 0, 0], [0, 0, 0, 0, 0, 0, 183, 0], [0, 94, 0, 0, 0, 0, 0, 0], [1050, 0, 0, 0, 0, 0, 0, 0], [0, 0, 123, 0, 0, 0, 0, 0], [0, 0, 0, 0, -133, 0, 0, 0], [0, 0, 0, 0, 0, 0, -207, 0], [0, 0, 0, -49, 0, 0, 0, 0], [0, -477, 0, 0, 0, 0, 0, 0], [0, 0, -45, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -23], [0, 0, 0, 0, 0, 264, 0, 0], [0, 0, 0, 0, 0, 0, 0, -135], [0, 50, 0, 0, 0, 0, 0, 0], [-804, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 255, 0, 0, 0], [0, 0, -63, 0, 0, 0, 0, 0], [-2612, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -354, 0, 0], [0, 0, 0, 0, 0, 0, 0, 71], [0, 612, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 441, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -165], [0, 0, -169, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 276, 0], [0, 0, 0, -114, 0, 0, 0, 0], [0, 504, 0, 0, 0, 0, 0, 0], [0, 0, -95, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 47], [0, 0, 0, 0, 0, 0, 0, 63], [0, 0, -107, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 560, 0, 0], [246, 0, 0, 0, 0, 0, 0, 0], [0, 0, -15, 0, 0, 0, 0, 0], [1180, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 571, 0, 0], [0, 0, 0, 0, 0, 0, 0, 319], [0, 0, 0, 0, 0, 0, 108, 0], [0, 0, 0, 0, 745, 0, 0, 0], [0, 0, 0, 0, 0, -473, 0, 0], [0, 0, 0, -26, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0, 0, 0], [0, 0, 0, 0, -887, 0, 0, 0], [0, 0, 0, 0, 0, 0, -72, 0], [0, 0, 0, 0, 0, -209, 0, 0], [0, 0, 0, -139, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -3], [0, 0, 0, 0, 0, 0, 100, 0], [762, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 361, 0, 0, 0], [0, 0, 0, 0, 0, 573, 0, 0], [0, 0, 0, 0, 0, 0, -300, 0], [0, 0, 0, 0, 1165, 0, 0, 0], [0, 0, 0, -49, 0, 0, 0, 0], [0, 143, 0, 0, 0, 0, 0, 0], [0, 0, 0, 160, 0, 0, 0, 0], [0, 0, -149, 0, 0, 0, 0, 0], [0, 0, 0, 0, -445, 0, 0, 0], [0, 0, 0, 0, 0, 0, 156, 0], [1108, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, 0, 205, 0, 0, 0, 0, 0], [0, 631, 0, 0, 0, 0, 0, 0], [0, 0, 167, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 1145, 0, 0, 0], [0, -630, 0, 0, 0, 0, 0, 0], [0, 0, 0, -131, 0, 0, 0, 0], [0, 0, 0, 0, 0, 542, 0, 0], [0, 929, 0, 0, 0, 0, 0, 0], [2280, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -5, 0], [0, 0, 0, 0, -341, 0, 0, 0], [0, 0, 0, 145, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 109], [0, 0, 0, 0, 0, -99, 0, 0], [0, 0, 0, 0, 0, 0, -95, 0], [0, -254, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -559, 0, 0], [0, 0, 0, 0, 0, 0, 0, 239], [0, 0, 0, 0, -483, 0, 0, 0], [0, 0, 0, -36, 0, 0, 0, 0], [0, 0, 0, 0, 0, -689, 0, 0], [0, 898, 0, 0, 0, 0, 0, 0]]
